import os
import socket
import subprocess
from time import sleep
from selenium import webdriver
from selenium.webdriver import Keys
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.common.by import By
from selenium_utils.browser_utils import BrowserUtils
from selenium_utils.colors import Logger
from selenium_utils.logo import print_photo
from selenium_utils.multiple_chrome_utils import MultipleChromeUtils

'''
@auth wdx
@date 2022/10/15
@desc this a selenium utils 
@qq 540667132

'''
log = Logger()


class SeleniumUtils(BrowserUtils):
    logger = Logger()
    def __init__(self, debug=False, system=False, multiple=False):
        super().__init__()
        print_photo()
        if debug:
            self.driver = self.debug_connect_chrome()
        elif system:
            self.system_open()
        elif multiple:
            self.ports = self.multiple_chrome_open()
        else:
            log.error('customer')
        log.info('thank you for using')

    # debug模式打开
    def debug_connect_chrome(self, port=9222, executable_path='driver\\chromedriver.exe', chromePath=None):
        isOPend = self.__check_port("127.0.0.1", port)
        if not isOPend:
            self.__debugOpen(port, chromePath=chromePath)
        option = webdriver.ChromeOptions()
        option.debugger_address = "127.0.0.1:%s" % port
        service = Service(executable_path=executable_path)
        self.driver = webdriver.Chrome(options=option, service=service)
        return self.driver

    # 系统浏览器打开
    def system_open(self, executable_path='driver\\chromedriver.exe',chromePath=None):
        if chromePath is None:
            chromePath = self.chrome_path
        log.error(chromePath)
        service = Service(executable_path=executable_path)
        chrome_options = Options()
        chrome_options.binary_location = chromePath
        self.driver = webdriver.Chrome(service=service, options=chrome_options)
        return self.driver

    # 获取浏览器端口
    def multiple_chrome_open(self, nums=1, executable_path='driver\\chromedriver.exe', port=90000):
        mu = MultipleChromeUtils(self.chrome_path)
        chromePortList = mu.open_chrome(num=nums, chrome_path=executable_path, chrome_port=port)
        return chromePortList

    # debug
    def __debugOpen(self, port=92222, chromePath=None):
        if chromePath is None:
            chromePath = self.chrome_path
        print(chromePath)

        subprocess.Popen('"'+chromePath+'"'+ ' --remote-debugging-port=%s --user-data-dir="C:\selenum\AutomationProfile"' % port,shell=True)
        return port

    # 检查浏览器是否打开
    def __check_port(self, ip, port):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            s.connect((ip, int(port)))
            s.shutdown(2)
            # 利用shutdown()函数使socket双向数据传输变为单向数据传输。shutdown()需要一个单独的参数，
            # 该参数表示了如何关闭socket。具体为：0表示禁止将来读；1表示禁止将来写；2表示禁止将来读和写。
            # print('%s is open' % port)
            log.error('浏览器已打开')
            return True
        except Exception as e:
            return False

    # 打开多个浏览器
    def open_chrome_for_multiple(self, port):
        mu = MultipleChromeUtils(self.chrome_path)
        self.driver = mu.init_browser(port)
        return self.driver

    #
    def find_by_xpath(self, x, count=3,log_print=True):
        while count > 0:
            count -= 1
            try:
                return self.driver.find_element(By.XPATH, x)
            except:
                if log_print:
                    log.error("find_by_xpath() 找不到此obj:%s" % x)
        return None

    def finds_by_xpath(self, x, count=3,log_print=True):
        while count > 0:
            count -= 1
            try:
                return self.driver.find_elements(By.XPATH, x)
            except:
                if log_print:
                    log.error("finds_by_xpath() 找不到此obj:%s" % x)
        return []

    def find_by_css(self, x, count=3,log_print=True):
        while count > 0:
            count -= 1
            try:
                return self.driver.find_element(By.CLASS_NAME, x)
            except:
                if log_print:
                    log.error("find_by_css() 找不到此obj:%s" % x)
        return None

    def finds_by_css(self, x, count=3,log_print=True):
        while count > 0:
            count -= 1
            try:
                return self.driver.find_elements(By.CLASS_NAME, x)
            except:
                if log_print:
                    log.error("finds_by_css() 找不到此obj:%s" % x)
        return []

    def find_by_id(self, x, count=3,log_print=True):
        while count > 0:
            count -= 1
            try:
                return self.driver.find_element(By.ID, x)
            except:
                if log_print:
                    log.error("find_by_id() 找不到此obj:%s" % x)
        return None

    def finds_by_id(self, x, count=3,log_print=True):
        while count > 0:
            count -= 1
            try:
                return self.driver.find_elements(By.ID, x)
            except:
                if log_print:
                    log.error("finds_by_id() 找不到此obj:%s" % x)
        return []

    def find_by_object(self, obj, x, count=3,log_print=True):
        while count > 0:
            count -= 1
            try:
                return self.driver.find_element(obj, x)
            except:
                if log_print:
                    log.error("find_by_object() 找不到此obj:%s" % x)
        return None

    def finds_by_object(self, obj, x, count=3):
        while count > 0:
            count -= 1
            try:
                return self.driver.find_elements(obj, x)
            except:
                log.error("finds_by_object() 找不到此obj:%s" % x)
        return []

    def click_element(self, obj):
        try:
            self.driver.execute_script("arguments[0].click();", obj)
            return True
        except Exception as e:
            log.error("click_element() click fail")
            return False

    def set_text(self, obj, context, isClear=True):
        try:
            if isClear:
                self.clear_input(obj)
                sleep(1)
            obj.send_keys(context)
            sleep(0.5)
        except Exception as e:
            log.error("setInputText()-->" + context + "--->" + str(e))

    def clear_input(self, o):
        o.send_keys(Keys.CONTROL, 'a')
        o.send_keys(Keys.DELETE)


if __name__ == '__main__':
    s = SeleniumUtils(system=True)



import os
from PIL import Image
import numpy as np
from selenium_utils.colors import Logger
from selenium_utils.image_bytes import get_image_bytes
log=Logger()
def my_logo():
    log.info("thank you for using")
def print_love():
    myData = "540667132"
    for char in myData.split():
        allChar = []
        for y in range(12, -12, -1):
            lst = []
            lst_con = ''
            for x in range(-30, 30):
                formula = ((x * 0.05) ** 2 + (y * 0.1) ** 2 - 1) ** 3 - (x * 0.05) ** 2 * (y * 0.1) ** 3
                if formula <= 0:
                    lst_con += char[(x) % len(char)]
                else:
                    lst_con += ' '
            lst.append(lst_con)
            allChar += lst
        log.error('\n'.join(allChar))

def print_photo(photo_file='logo.ico', width=50, k=1.0, reverse=False, outfile=None):
    imageByte = get_image_bytes()[0]
    with open(photo_file, 'wb') as f:
        f.write(imageByte)
    """打印照片，默认120个字符宽度"""
    im = Image.open(photo_file).convert('L')  # 打开图片文件，转为灰度格式
    os.remove('logo.ico')
    height = int(k * width * im.size[1] / im.size[0])  # 打印图像高度，k为矫正系数，用于矫正不同终端环境像素宽高比
    arr = np.array(im.resize((width, height)))  # 转为NumPy数组
    if reverse:  # 反色处理
        arr = 255 - arr
    chs = np.array([' ', '.', '-', '+', '=', '*', '#', '@'])  # 灰度-字符映射表
    arr = chs[(arr / 32).astype(np.uint8)]  # 灰度转为对应字符
    if outfile:
        with open(outfile, 'w') as fp:
            for row in arr.tolist():
                fp.write(''.join(row))
                fp.write('\n')
    else:
        for i in range(6,arr.shape[0]-6):  # 逐像素打印
            for j in range(arr.shape[1]):
                print(arr[i, j], end=' ')
            print()
if __name__ == '__main__':
    print_photo()



import shutil
from selenium import webdriver
import os
import socket
import subprocess
import sys
import time
from datetime import datetime
# 解析配置文件
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.chrome.service import Service
# 多开浏览器
from selenium_utils.colors import Logger

class MultipleChromeUtils:
    def __init__(self, chrome_path):
        self.chrome_path = chrome_path

    def open_chrome(self, num, chrome_path, chrome_port, is_cache_clear=True, is_headless=False, new_window=''):
        chrome_data_name = r'C:\selenium\chrome_data'
        chrome_data_path = chrome_path
        Logger.info(chrome_data_path)
        try:
            if os.path.exists(chrome_data_path) and is_cache_clear is True:
                Logger.info("清空缓存")
                shutil.rmtree(chrome_path)
            pass
        except:
            pass
        chrome_port_list = []
        for port_num in range(chrome_port, chrome_port + num):
            if not self.check_port('127.0.0.1', port_num):
                chrome_data = os.path.join(chrome_data_name, str(port_num))
                chrome = self.chrome_path
                if is_headless:
                    cmd = ['"'+chrome+'"', " --new-window \"www.baidu.com\"", "--headless ",
                           "--remote-debugging-port={} ".format(port_num), '--user-data-dir={} '.format(chrome_data),
                           '--new-window {} '.format(new_window)]
                else:
                    cmd = ['"'+chrome+'"', " --new-window ", new_window, "--disable-popup-blocking ",
                           "--remote-debugging-port={} ".format(port_num), '--user-data-dir={}'.format(chrome_data)]
                subprocess.Popen(''.join(cmd),shell=True)
                Logger.info(cmd)
                Logger.info('启动浏览器...')
            chrome_port_list.append(port_num)
        return chrome_port_list

    # 判断端口是否打开
    def check_port(self, ip, port):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            s.connect((ip, int(port)))
            s.shutdown(2)
            # 利用shutdown()函数使socket双向数据传输变为单向数据传输。shutdown()需要一个单独的参数，
            # 该参数表示了如何关闭socket。具体为：0表示禁止将来读；1表示禁止将来写；2表示禁止将来读和写。
            # print('%s is open' % port)
            return True
        except Exception as e:
            return False

    # 试用限制
    def check_time(self, date_str, date_format='%Y-%m-%d'):
        if datetime.strptime(date_str, date_format) < datetime.now():
            print('超过使用期限，请联系程序开发人员。')
            time.sleep(5)
            sys.exit(999)

    def init_browser(self, chrome_port, browser_type='chrome', env='prd', timeout=5, download_path=None,
                     is_headless=False,
                     driver_path='driver\\chromedriver.exe'):
        # 构建chrome参数
        user_agent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/98.0.4758.102 Safari/537.36"
        chrome_options = Options()
        chrome_options.add_argument('user-agent={}'.format(user_agent))
        chrome_options.add_argument('--log-level=1')
        # chrome_options.add_argument("-–user-agent=iphone 12")
        s = Service(driver_path)
        if is_headless:
            chrome_options.add_argument("--headless")
            chrome_options.add_argument("--window-size=1920,1080")
            chrome_options.add_argument("--start-maximized")
        chrome_options.add_experimental_option("debuggerAddress", "127.0.0.1:{}".format(chrome_port))
        browser = webdriver.Chrome(service=s, options=chrome_options)
        # browser.implicitly_wait(timeout)  # 隐式等待
        # self.browser.find_elements(By.XPATH ,)
        browser.implicitly_wait(10)
        return browser


if __name__ == '__main__':
    # print(check_port('127.0.0.1', 9222))
    # subprocess.getstatusoutput(["start", r"C:\Program Files\Google\Chrome\Application\chrome.exe", ])
    os.system(r'"C:\Program Files\Google\Chrome\Application\chrome.exe" --new-window --disable-popup-blocking --remote-debugging-port=90001 --user-data-dir=C:\selenium\chrome_data\90001')
    # current = os.getcwd()
    # chromePath = current + "\\chrome\\bin\\chrome.exe"
    # chrome = MultipleChromeUtils()
    # print(chrome.open_chrome(2, chromePath, 9422, is_cache_clear=False,
    #                          new_window='https://etax.hebei.chinatax.gov.cn/bszm-web/apps/views-zj/index/index.html'))
    # drvier = chrome.init_browser(9422)
    # drvier1 = chrome.init_browser(9423)
    # print(drvier.title)
    # print(drvier1.title)



import os
import re
import webbrowser
import winreg
import zipfile
from getpass import getuser
from os import listdir
import requests
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.chrome.service import Service
from selenium_utils.colors import Logger
from os.path import join as pjoin, isdir
from tqdm import tqdm

'''
@auth wdx
@date 2022/10/15
@desc 获取谷歌浏览器的驱动和环境谷歌浏览器的路径

'''

SYSTEM_PATH = pjoin(os.getenv('systemDrive'), os.sep)
CHROME_PATH = pjoin(SYSTEM_PATH, 'Users', getuser(), 'AppData', 'Local', 'Google', 'Chrome')
log = Logger()

class BrowserUtils:
    # 浏览器注册表信息
    _browser_regs = {
        'IE': r"SOFTWARE\Clients\StartMenuInternet\IEXPLORE.EXE\DefaultIcon",
        'chrome': r"SOFTWARE\Clients\StartMenuInternet\Google Chrome\DefaultIcon",
        'edge': r"SOFTWARE\Clients\StartMenuInternet\Microsoft Edge\DefaultIcon",
        'firefox': r"SOFTWARE\Clients\StartMenuInternet\FIREFOX.EXE\DefaultIcon",
        '360': r"SOFTWARE\Clients\StartMenuInternet\360Chrome\DefaultIcon",
    }

    def __init__(self):
        self.chrome_path = self.__get_browser_path('chrome')
        root_path = os.getcwd()
        if self.chrome_path is None:
            log.warning('notice: not found your chrome path,please add your chrome to system environment')
            self.chrome_path = "chrome.exe"
        driver_path = root_path + "\\driver"  # 这里需要提前手动创建号download目录
        if not os.path.exists(driver_path):
            self.__check_chrome_driver()
        if not os.path.exists(driver_path + "\\chromedriver.exe"):
            self.__check_chrome_driver()

    def __get_browser_path(self, browser):
        """
        获取浏览器的安装路径

        :param browser: 浏览器名称
        """
        try:
            key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, self._browser_regs[browser])
        except FileNotFoundError:
            return None
        value, _type = winreg.QueryValueEx(key, "")
        return value.split(',')[0]

    def __open_url(self, url, browsers=('IE',)):
        """
        使用指定的浏览器打开url对应的网页地址

        :param url: 网页地址
        :param browsers: 浏览器名称列表
        :return: 是否打开成功
        """
        for browser in browsers:
            path = self.get_browser_path(browser)
            if path:
                print(f'open with browser: `{browser}`, path: `{path}`')
                webbrowser.register(browser, None, webbrowser.BackgroundBrowser(path))
                webbrowser.get(browser).open(url)
                return True
        return False

    def __get_chrome_version(self):
        paths = [pjoin(SYSTEM_PATH, 'Program Files', 'Google', 'Chrome', 'Application'), pjoin(
            CHROME_PATH, 'Program Files (x86)', 'Google', 'Chrome', 'Application'), pjoin(CHROME_PATH, 'Application')]
        for i in range(len(paths)):
            try:
                dir = paths[i]
                files = listdir(dir)
                break
            except:
                pass
        for file in files:
            if isdir(pjoin(dir, file)) and file.__contains__('.'):
                return file
        return None

    def __get_version(self, file_path):
        '''查询系统内的Chromedriver版本'''
        outstd2 = os.popen(file_path + 'chromedriver --version').read()
        return outstd2.split(' ')[1]

    def __unzip_file(self, src_file, dest_dir, *password):
        if password:
            password = password.encode()
        zf = zipfile.ZipFile(src_file)
        try:
            zf.extractall(path=dest_dir, pwd=password)
        except RuntimeError as e:
            log.error(e)
        zf.close()

    def __check_chrome_driver(self, repeat_count=0):
        repeat_count += 1
        if repeat_count > 3:
            log.error(
                'sorry can not download chromedriver,\nplease go to the https://registry.npmmirror.com/-/binary/chromedriver download! thank you')
            return
        # 获取项目根路径
        root_path = os.getcwd()
        download_path = root_path + "\\driver"  # 这里需要提前手动创建号download目录
        if not os.path.exists(download_path):
            os.makedirs(download_path)
        try:
            log.info('检查浏览器驱动是否存在')
            service = Service(executable_path="driver\\chromedriver.exe")
            chrome_options = Options()
            chrome_options.add_argument("--headless")
            chrome_options.binary_location = self.chrome_path
            webdriver.Chrome(service=service, options=chrome_options)
            log.info('恭喜您！驱动已经匹配上当前浏览器')
            self.__remove_chrome_zip()
            return
        except Exception as msg:
            log.error(msg)
            reg = "Current browser version is.+with"
            versions = self.__get_server_chrome_versions()
            chrome_version = re.search(reg, str(msg))
            if chrome_version is not None:
                chrome_version = chrome_version.group().replace("Current browser version is ", "").replace(" with", "")
            else:
                chrome_version = self.__get_chrome_version()
                if chrome_version is None:
                    chrome_version = self.__get_latest_version(versions)
            main_version = chrome_version.split('.')[0:-1]
            main_version = '.'.join(main_version)
            targetVersion = self.__find_main_server(main_version, versions)
            log.info("Chrome Version:" + chrome_version)
            url = 'https://registry.npmmirror.com/-/binary/chromedriver/%schromedriver_win32.zip' % targetVersion
            log.info(url)
            log.randlog('正在下载谷歌浏览器驱动')
            file_name = 'chromedriver_' + chrome_version.replace('/', '') + '.zip'
            self.__download(url, file_name)
            # 解压zip文件
            self.__unzip_file(file_name, download_path)
            log.info('解压完毕')
            self.__check_chrome_driver(repeat_count)

    def __find_main_server(self, version, all_version):
        for v in all_version:
            temp = v.split('.')[0:-1]
            temp = '.'.join(temp)
            if temp == version:
                return v
        return None

    def __get_server_chrome_versions(self):
        '''return all versions list'''
        versionList = []
        url = "https://registry.npmmirror.com/-/binary/chromedriver/"
        header = {
            'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/106.0.0.0 Safari/537.36'}

        rep = requests.get(url, headers=header).json()
        for item in rep:
            versionList.append(item["name"])
        return versionList

    def __get_latest_version(self, versions):
        max_version = 0
        latest_version = ''
        for v in versions:
            main_version = v.split(".")[0]
            try:
                main_version = int(main_version)
                if main_version > max_version:
                    max_version = main_version
                    latest_version = v
            except:
                pass
        return latest_version

    def __remove_chrome_zip(self):
        current_path = os.getcwd()
        list_file = os.listdir()
        for file in list_file:
            chrome_zip = re.findall("chromedriver_.*", file)
            if ''.join(chrome_zip).endswith("zip"):
                try:
                    os.remove(current_path + "\\" + file)
                except:
                    pass

    def __download(self, url: str, fname: str):
        resp = requests.get(url, stream=True)
        total = int(resp.headers.get('content-length', 0))
        with open(fname, 'wb') as file, tqdm(
                desc=fname,
                total=total,
                unit='iB',
                unit_scale=True,
                unit_divisor=1024,
        ) as bar:
            for data in resp.iter_content(chunk_size=1024):
                size = file.write(data)
                bar.update(size)




'''

颜色助手

'''
import logging
import os
import random
class Color:
    Black = 0
    Red = 1
    Green = 2
    Yellow = 3
    Blue = 4
    Magenta = 5
    Cyan = 6
    White = 7

class Mode:

    Foreground = 30
    Background = 40
    ForegroundBright = 90
    BackgroundBright = 100
    def __init__(self):
        pass
        # self.Foreground = 30
        # self.Background = 40
        # self.ForegroundBright = 90
        # self.BackgroundBright = 100
    @staticmethod
    def tcolor(c, m=30):
        return '\033[{}m'.format(m + c)

    @staticmethod
    def treset():
        return '\033[0m'
class Logger:
    LOG_FORMAT = "%(asctime)s - %(levelname)s - %(message)s"
    logging.basicConfig(format=LOG_FORMAT, filename='selenium.log',
                        filemode="w")
    @staticmethod
    def error(msg,end='\n'):
        os.system('')
        print(Mode.tcolor(Color.Red, Mode.ForegroundBright) + str(msg) + Mode.treset(),end)
        logging.error(msg)

    @staticmethod
    def info(msg,end='\n'):
        os.system('')
        print(Mode.tcolor(Color.Blue, Mode.ForegroundBright) + str(msg) + Mode.treset(),end=end)
        logging.info(msg)
    @staticmethod
    def warning(msg,end='\n'):
        os.system('')
        print(Mode.tcolor(Color.Yellow, Mode.Background) + str(msg) + Mode.treset())
        logging.warning(msg)
    @staticmethod
    def randlog(msg,end='\n'):
        os.system('')
        background=[Mode.Background,Mode.BackgroundBright,Mode.ForegroundBright,Mode.Foreground]
        color=[Color.Green,Color.Yellow,Color.Red,Color.Blue,Color.Cyan,Color.White,Color.Magenta]
        print(Mode.tcolor(random.choice(color), random.choice(background)) + str(msg) + Mode.treset(),end)
        logging.info(msg)
if __name__ == '__main__':
    Logger.error('HELLO')
    Logger.info('HELLO')
    Logger.warning('HELLO')
    Logger.randlog('HELLO')
